@@ -30,12 +30,11 @@ monitor the service. Eg:
   cmd: pgrep memcached
   refresh: 30
   service_port: 11211
+  service_hostname: 123.123.123.123
   zookeeper_path: /services/prod-uswest1-mc
-  zookeeper_data: foo=bar
+  zookeeper_data: { "foo": "bar", "bar": "foo" }
 
 Copyright 2012 Nextdoor Inc.
-References: http://code.activestate.com/recipes/66012/
-Advanced Programming in the Unix Environment by W. Richard Stevens
 """
 
 __author__ = 'matt@nextdoor.com (Matt Wise)'
@@ -46,6 +45,7 @@ import socket
 import subprocess
 import threading
 import time
+import json
 import signal
 import ConfigParser
 import logging
@@ -54,6 +54,7 @@ import os
 
 # Get our ServiceRegistry class
 from ndServiceRegistry import KazooServiceRegistry as ServiceRegistry
+from ndServiceRegistry import exceptions
 
 # Our default variables
 from version import __version__ as VERSION
@@ -177,32 +178,54 @@ class WatcherDaemon(threading.Thread):
         for service in self._config.sections():
             w = self._get_watcher(service)
 
+            # Gather up the config data for our section into a few local
+            # variables so that we can shorten the statements below.
+            command = self._config.get(service, 'cmd')
+            service_port = self._config.get(service, 'service_port')
+            zookeeper_path = self._config.get(service, 'zookeeper_path')
+            refresh = self._config.get(service, 'refresh')
+
+            # Gather our optional parameters. If they don't exist, set
+            # some reasonable default.
+            try:
+                zookeeper_data = self._parse_data(
+                    self._config.get(service, 'zookeeper_data'))
+            except:
+                zookeeper_data = {}
+
+            try:
+                service_hostname = self._config.get(service, 'service_hostname')
+            except:
+                service_hostname = socket.getfqdn()
+
             if w:
                 # Certain fields cannot be changed without destroying the
                 # object and its registration with Zookeeper.
-                if w._service_port != self._config.get(service, 'service_port') or \
-                   w._path != self._config.get(service, 'zookeeper_path'):
+                if w._service_port != service_port or \
+                    w._service_hostname != service_hostname or \
+                        w._path != zookeeper_path:
                     w.stop()
                     w = None
 
             if w:
                 # We already have a watcher for this service. Update its
                 # object data, and let it keep running.
-                w.set(command=self._config.get(service, 'cmd'),
-                      data=self._parse_data(self._config.get(service, 'zookeeper_data')),
-                      refresh=self._config.get(service, 'refresh'))
+                w.set(command=command,
+                      data=zookeeper_data,
+                      refresh=refresh)
 
             # If there's still no 'w' returned (either _get_watcher failed, or
             # we noticed that certain un-updatable fields were changed, then
             # create a new object.
             if not w:
                 w = ServiceWatcher(registry=self._sr,
-                                  service=service,
-                                  service_port=self._config.get(service, 'service_port'),
-                                  command=self._config.get(service, 'cmd'),
-                                  path=self._config.get(service, 'zookeeper_path'),
-                                  data=self._parse_data(self._config.get(service, 'zookeeper_data')),
-                                  refresh=self._config.get(service, 'refresh'))
+                                   service=service,
+                                   service_port=service_port,
+                                   service_hostname=service_hostname,
+                                   command=command,
+                                   path=zookeeper_path,
+                                   data=zookeeper_data,
+                                   refresh=refresh)
                 self._watchers.append(w)
 
         # Check if any watchers need to be destroyed because they're no longer
@@ -222,20 +245,26 @@ class WatcherDaemon(threading.Thread):
     def _parse_data(self, data):
         """Convert a string of data from ConfigParse into our dict.
 
-        If any options are supplied to the zookeeper_data field, then we add
-        them to our node registration. The values must be comma-separated
-        and equals-separated. eg:
-     
-        foo=bar,abc=123
+        The zookeeper_data field supports one of two types of fields. Either
+        a single key=value string, or a JSON-formatted set of key=value
+        pairs:
+
+            zookeeper_data: foo=bar
+            zookeeper_data: foo=bar, bar=foo
+            zookeeper_data: { "foo": "bar", "bar": "foo" }
 
         Args:
-            data: String representing key=value pairs comma separated"""
-        data_dict = {}
-        for pair in data.split(','):
-            if pair.split('=').__len__() == 2:
-                key = pair.split('=')[0]
-                value = pair.split('=')[1]
-                data_dict[key] = value
+            data: String representing data above"""
+
+        try:
+            data_dict = json.loads(data)
+        except:
+            data_dict = {}
+            for pair in data.split(','):
+                if pair.split('=').__len__() == 2:
+                    key = pair.split('=')[0]
+                    value = pair.split('=')[1]
+                    data_dict[key] = value
         return data_dict
 
     def run(self):
@@ -261,17 +290,17 @@ class ServiceWatcher(threading.Thread):
     LOGGER = 'WatcherDaemon.ServiceWatcher'
 
     def __init__(self, registry, service, service_port, command, path, data,
-                 name=socket.getfqdn(), refresh=15):
+                 service_hostname, refresh=15):
         """Initialize the object and begin monitoring the service."""
         # Initiate our thread
         super(ServiceWatcher, self).__init__()
 
         self._sr = registry
-        self._name = name
         self._service = service
         self._service_port = service_port
+        self._service_hostname = service_hostname
         self._path = path
-        self._fullpath = '%s/%s:%s' % (path, name, service_port)
+        self._fullpath = '%s/%s:%s' % (path, service_hostname, service_port)
         self.set(command, data, refresh)
         self.log = logging.getLogger('%s.%s' % (self.LOGGER, self._service))
         self.log.debug('Initializing...')
@@ -314,7 +343,7 @@ class ServiceWatcher(threading.Thread):
 
                 if ret == 0:
                     # If the command was successfull...
-                    self.log.info('[%s] returned successfully' % self._command)
+                    self.log.debug('[%s] returned successfull' % self._command)
                     self._update(state=True)
                 else:
                     # If the command failed...
@@ -352,7 +381,7 @@ class ServiceWatcher(threading.Thread):
             self.log.debug('[%s] sucessfully updated path %s with state %s' %
                           (self._service, self._fullpath, state))
             return True
-        except Exception, e:
+        except exceptions.NoConnection, e:
             self.log.warn('[%s] could not update path %s with state %s: %s' %
                          (self._service, self._fullpath, state, e))
             return False
